package com.huimv.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.admin.common.utils.DataUill;
import com.huimv.admin.common.utils.Result;
import com.huimv.admin.common.utils.ResultCode;
import com.huimv.admin.entity.EnvDeviceOnline;
import com.huimv.admin.entity.vo.EnvDeviceOnlineVo;
import com.huimv.admin.mapper.EnvDeviceOnlineMapper;
import com.huimv.admin.service.IEnvDeviceOnlineService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-03-24
 */
@Service
public class EnvDeviceOnlineServiceImpl extends ServiceImpl<EnvDeviceOnlineMapper, EnvDeviceOnline> implements IEnvDeviceOnlineService {

    @Resource
    private EnvDeviceOnlineMapper envDeviceOnlineMapper;

    @Override
    public Result listDeviceStatus(Map<String, String> paramsMap) {
        Date timesMonthmorning = DataUill.getTimesMonthmorning();
        String farmId = paramsMap.get("farmId");
        Map resultMap = new HashMap();

        QueryWrapper<EnvDeviceOnline> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id",farmId).ge("creat_time",timesMonthmorning);
        List<EnvDeviceOnlineVo>  envDeviceOnlines = envDeviceOnlineMapper.getDeviceStatus(queryWrapper);

        BigDecimal currentStatus = envDeviceOnlineMapper.getStatus(queryWrapper);
        resultMap.put("data",envDeviceOnlines);
        resultMap.put("currentValue",currentStatus);
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); //格式化时间
        //上个月
        //获取上个月的第一天
        Calendar cal_1 = Calendar.getInstance();//获取当前日期
        cal_1.add(Calendar.MONTH, -1);
        cal_1.set(Calendar.DAY_OF_MONTH, 1);//设置为1号
        cal_1.set(Calendar.HOUR_OF_DAY, 0);
        cal_1.set(Calendar.MINUTE, 0);
        cal_1.set(Calendar.SECOND, 0);
        String firstDay = format.format(cal_1.getTime());
        //获取上个月的最后一天
        Calendar cal_2 = Calendar.getInstance();
        cal_2.set(Calendar.DAY_OF_MONTH, 0);//设置为1号,当前日期既为本月第一天
        cal_2.set(Calendar.HOUR_OF_DAY, 23);
        cal_2.set(Calendar.MINUTE, 59);
        cal_2.set(Calendar.SECOND, 59);
        String lastDay = format.format(cal_2.getTime());
        QueryWrapper<EnvDeviceOnline> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_id",farmId).between("creat_time",firstDay,lastDay);
        BigDecimal lastStatus = envDeviceOnlineMapper.getStatus(wrapper);
        if (ObjectUtil.isNull(lastStatus)){
            resultMap.put("lastValue",0);
            resultMap.put("gap",100);
            return new Result(ResultCode.SUCCESS,resultMap);
        }
        resultMap.put("lastValue",lastStatus);
        double gap = ((currentStatus.doubleValue() - lastStatus.doubleValue()) / lastStatus.doubleValue()) * 100;
        if (ObjectUtil.isNull(gap)){
            gap = 0.0;
        }
        BigDecimal bg = new BigDecimal(gap).setScale(2, RoundingMode.UP);
        resultMap.put("gap",bg);
        return new Result(ResultCode.SUCCESS,resultMap) ;
    }
}
